@@ -7,13 +7,11 @@
 """
 
 import os
+import tempfile
+
 from MoinMoin import config
-from MoinMoin.util import filesys
+from MoinMoin.util import filesys, lock
 
-locking = 1
-if locking:
-    from MoinMoin.util import lock
-    
 class CacheEntry:
     def __init__(self, request, arena, key):
         """ init a cache entry
@@ -23,18 +21,16 @@ class CacheEntry:
             @param key: under which key we access the cache content
         """
         self.request = request
+        self.key = key
         if isinstance(arena, str):
             self.arena_dir = os.path.join(request.cfg.cache_dir, arena)
             filesys.makeDirs(self.arena_dir)
         else: # arena is in fact a page object
-            cache_dir = None
             self.arena_dir = arena.getPagePath('cache', check_create=1)
-        self.key = key
-        if locking:
-            self.lock_dir = os.path.join(self.arena_dir, '__lock__')
-            self.rlock = lock.ReadLock(self.lock_dir, 60.0)
-            self.wlock = lock.WriteLock(self.lock_dir, 60.0)
-        
+        self.lock_dir = os.path.join(self.arena_dir, '__lock__')
+        self.rlock = lock.LazyReadLock(self.lock_dir, 60.0)
+        self.wlock = lock.LazyWriteLock(self.lock_dir, 60.0)
+
     def _filename(self):
         return os.path.join(self.arena_dir, self.key)
 
@@ -48,8 +44,9 @@ class CacheEntry:
             return 0
 
     def needsUpdate(self, filename, attachdir=None):
-        if not self.exists():
-            return 1
+        # following code is not necessary. will trigger exception and give same result
+        #if not self.exists():
+        #    return 1
 
         try:
             ctime = os.path.getmtime(self._filename())
@@ -58,7 +55,7 @@ class CacheEntry:
             return 1
 
         needsupdate = ftime > ctime
-        
+
         # if a page depends on the attachment dir, we check this, too:
         if not needsupdate and attachdir:
             try:
@@ -66,57 +63,48 @@ class CacheEntry:
             except os.error:
                 ftime2 = 0
             needsupdate = ftime2 > ctime
-                
+
         return needsupdate
 
-    def copyto(self, filename):
-        import shutil
-        if not locking or locking and self.wlock.acquire(1.0):
-            try:
-                shutil.copyfile(filename, self._filename())
-                try:
-                    os.chmod(self._filename(), 0666 & config.umask)
-                except OSError:
-                    pass
-            finally:
-                if locking:
-                    self.wlock.release()
-        else:
-            self.request.log("Can't acquire write lock in %s" % self.lock_dir)
-
     def update(self, content, encode=False):
+        fname = self._filename()
         if encode:
             content = content.encode(config.charset)
-        if not locking or locking and self.wlock.acquire(1.0):
+        if self.wlock.acquire(1.0):
             try:
-                f = open(self._filename(), 'wb')
-                f.write(content)
-                f.close()
-                try:
-                    os.chmod(self._filename(), 0666 & config.umask)
-                except OSError:
-                    pass
+                # we do not write content to old inode, but to a new file
+                # se we don't need to lock when we just want to read the file
+                # (at least on POSIX, this works)
+                tmp_handle, tmp_fname = tempfile.mkstemp('.tmp', self.key, self.arena_dir)
+                os.write(tmp_handle, content)
+                os.close(tmp_handle)
+                # this is either atomic or happening with real locks set:
+                filesys.rename(tmp_fname, fname)
             finally:
-                if locking:
-                    self.wlock.release()
+                self.wlock.release()
         else:
             self.request.log("Can't acquire write lock in %s" % self.lock_dir)
 
     def remove(self):
-        try:
-            os.remove(self._filename())
-        except OSError:
-            pass
+        if self.wlock.acquire(1.0):
+            try:
+                try:
+                    os.remove(self._filename())
+                except OSError:
+                    pass
+            finally:
+                self.wlock.release()
+        else:
+            self.request.log("Can't acquire write lock in %s" % self.lock_dir)
 
     def content(self, decode=False):
-        if not locking or locking and self.rlock.acquire(1.0):
+        if self.rlock.acquire(1.0):
             try:
                 f = open(self._filename(), 'rb')
                 data = f.read()
                 f.close()
             finally:
-                if locking:
-                    self.rlock.release()
+                self.rlock.release()
         else:
             self.request.log("Can't acquire read lock in %s" % self.lock_dir)
         if decode:
