@@ -106,3 +106,8 @@ class ConfigurationError(FatalError):
 
 class InternalError(FatalError):
     """ Raise when internal fatal error is found """
+
+class NoConfigMatchedError(Exception):
+    """ we didn't find a configuration for this URL """
+    pass
+
