@@ -165,6 +165,11 @@ class MoinWriter(html4css1.Writer):
                                  self)
         self.document.walkabout(visitor)
         self.visitor = visitor
+        # Docutils 0.5.0 and later require the writer to have the visitor 
+        # attributes.
+        if (hasattr(html4css1.Writer, 'visitor_attributes')):
+            for attr in html4css1.Writer.visitor_attributes:
+                setattr(self, attr, getattr(visitor, attr))
         self.output = html_escape_unicode(visitor.astext())
 
 class Parser:
@@ -189,6 +194,7 @@ class Parser:
                 'file_insertion_enabled': 0,
                 'raw_enabled': 0,
                 'stylesheet_path': '',
+                'template': '',
             }
         )
 
@@ -370,7 +376,8 @@ class MoinTranslator(html4css1.HTMLTranslator):
                     node['classes'].append(prefix)
             else:
                 # Default case - make a link to a wiki page.
-                page = MoinMoin.Page.Page(self.request, refuri)
+                page = MoinMoin.Page.Page(self.request,
+                                          wikiutil.AbsPageName(self.request, self.formatter.page.page_name, refuri))
                 node['refuri'] = page.url(self.request)
                 if not page.exists():
                     node['classes'].append('nonexistent')
@@ -567,7 +574,10 @@ class MoinDirectives:
             state_machine.insert_input(lines, 'MoinDirectives')
         return
 
-    include.content = True
+    include.has_content = include.content = True
+    include.option_spec = {}
+    include.required_arguments = 1
+    include.optional_arguments = 0
 
     # Add additional macro directive.
     # This allows MoinMoin macros to be used either by using the directive
@@ -589,7 +599,10 @@ class MoinDirectives:
             return [ref]
         return
 
-    macro.content = True
+    macro.has_content = macro.content = True
+    macro.option_spec = {}
+    macro.required_arguments = 1
+    macro.optional_arguments = 0
 
 if ErrorParser: # fixup in case of missing docutils
     Parser = ErrorParser
